@@ -1,20 +1,8 @@
 package Google::RestApi::SheetsApi4;
 
-use strict;
-use warnings;
+our $VERSION = '0.6.1';
 
-our $VERSION = '0.3.1';
-
-use 5.010_000;
-
-use autodie;
-use Type::Params qw(compile compile_named);
-use Types::Standard qw(Str StrMatch HashRef HasMethods Any slurpy);
-use YAML::Any qw(Dump);
-
-no autovivification;
-
-use Google::RestApi::Utils qw(named_extra);
+use Google::RestApi::Setup;
 
 use aliased 'Google::RestApi::DriveApi3';
 use aliased 'Google::RestApi::SheetsApi4::Spreadsheet';
@@ -28,8 +16,6 @@ use constant {
   Worksheet_Uri      => "[#&]gid=([0-9]+)",
 };
 
-do 'Google/RestApi/logger_init.pl';
-
 sub new {
   my $class = shift;
 
@@ -65,7 +51,7 @@ sub create_spreadsheet {
     _extra_ => slurpy Any,
   );
   my $p = named_extra($check->(@_));
-  $p->{title} || $p->{name} or die "Either 'title' or 'name' should be supplied";
+  $p->{title} || $p->{name} or LOGDIE "Either 'title' or 'name' should be supplied";
   $p->{title} ||= $p->{name};
   delete $p->{name};
 
@@ -76,7 +62,7 @@ sub create_spreadsheet {
     },
   );
   for (qw(spreadsheetId spreadsheetUrl properties)) {
-    $result->{$_} or die "No '$_' returned from creating spreadsheet";
+    $result->{$_} or LOGDIE "No '$_' returned from creating spreadsheet";
   }
 
   return $self->open_spreadsheet(
@@ -115,7 +101,7 @@ sub delete_all_spreadsheets {
   my @spreadsheets = grep { $_->{name} eq $name; } @{ $spreadsheets->{files} };
   DEBUG(sprintf("Deleting %d spreadsheets for name '$name'", scalar @spreadsheets));
   $self->delete_spreadsheet($_->{id}) foreach (@spreadsheets);
-  return;
+  return scalar @spreadsheets;
 }
 
 sub spreadsheets {
@@ -219,8 +205,8 @@ Google::RestApi::SheetsApi4 - API to Google Sheets API V4.
  # now collect the ranges into a group and send the values via batch:
  $rg = $sheet->range_group($tax, $salary);
  $rg->submit_values();
- # bold and italicize both cells:
- $rg->bold()->italic()->submit_requests();
+ # bold and italicize both cells, and put a solid border around each one:
+ $rg->bold()->italic()->bd_solid()->submit_requests();
 
  # tie ranges to a hash:
  $row = $ws0->tie_cells({id => 'A2'}, {name => 'B2'});
@@ -355,7 +341,8 @@ Deletes a spreadsheet from Google Drive.
 =item delete_all_spreadsheets(spreadsheet_name<string>);
 
 Deletes all spreadsheets with the given name from Google Drive. Note that
-Google Sheets allows more than one spreadsheet to have the same name.
+Google Sheets allows more than one spreadsheet to have the same name. Returns
+the number of spreadsheets deleted.
 
  spreadsheet_name: The name of the spreadsheets you want to delete.
 
