@@ -1,22 +1,10 @@
 package Google::RestApi::SheetsApi4::Range::All;
 
-use strict;
-use warnings;
-
-our $VERSION = '0.3.1';
-
-use 5.010_000;
-
-use autodie;
-use Type::Params qw(compile_named);
-use Types::Standard qw(HasMethods);
-
-no autovivification;
+our $VERSION = '0.6.1';
 
+use Google::RestApi::Setup;
 use parent 'Google::RestApi::SheetsApi4::Range';
 
-do 'Google/RestApi/logger_init.pl';
-
 sub new {
   my $class = shift;
 
