@@ -1,56 +1,43 @@
 package Google::RestApi::SheetsApi4::Range::Cell;
 
-use strict;
-use warnings;
+our $VERSION = '0.6.1';
 
-our $VERSION = '0.3.1';
-
-use 5.010_000;
-
-use autodie;
-use Carp qw(cluck);
-use Type::Params qw(compile compile_named);
-use Types::Standard qw(Str InstanceOf Any slurpy);
-use YAML::Any qw(Dump);
-
-no autovivification;
-
-use Google::RestApi::Utils qw(named_extra);
+use Google::RestApi::Setup;
 
 use parent 'Google::RestApi::SheetsApi4::Range';
 
-do 'Google/RestApi/logger_init.pl';
-
+# make sure the translated range refers to a single cell (no ':').
 sub range {
   my $self = shift;
   return $self->{normalized_range} if $self->{normalized_range};
   my $range = $self->SUPER::range(@_);
-  die "Unable to translate '$range' into a worksheet cell"
+  LOGDIE "Unable to translate '$range' into a worksheet cell"
     if $range =~ /:/;
   return $range;
 }
 
 sub values {
   my $self = shift;
-  my $p = _update_values(@_);
+  state $check = compile_named(
+    values => Str, { optional => 1 },
+    _extra_ => slurpy Any,
+  );
+  my $p = named_extra($check->(@_));
+  $p->{values} = [[ $p->{values} ]] if defined $p->{values};
   my $values = $self->SUPER::values(%$p);
-  return $values->[0]->[0];
+  return defined $values ? $values->[0]->[0] : undef;
 }
 
 sub batch_values {
   my $self = shift;
-  my $p = _update_values(@_);
-  return $self->SUPER::batch_values(%$p);
-}
 
-sub _update_values {
   state $check = compile_named(
-    values  => Str, { optional => 1 },
-    _extra_ => slurpy Any,
+    values => Str, { optional => 1 },
   );
-  my $p = named_extra($check->(@_));
-  $p->{values} = [[$p->{values}]] if defined $p->{values};
-  return $p;
+  my $p = $check->(@_);
+
+  $p->{values} = [[ $p->{values} ]] if $p->{values};
+  return $self->SUPER::batch_values(%$p);
 }
 
 sub cell { shift; }
