@@ -5,22 +5,23 @@ use warnings;
 
 # use Carp::Always;
 use FindBin;
+use lib "$FindBin::RealBin/../../lib";
 use lib "$FindBin::RealBin/../../../lib";
-use lib "$FindBin::RealBin/../../integration";
 
 use YAML::Any qw(Dump);
-use Utils qw(init_logger message start end end_go show_api);
+use Utils qw(:all);
 
 init_logger();
 
 my $name = "Sheet1";
-my $spreadsheet_name = $Utils::spreadsheet_name;
-my $sheets = Utils::sheets_api(post_process => \&show_api);
+my $spreadsheet_name = spreadsheet_name();
+my $sheets = sheets_api(post_process => \&show_api);
 
 start("Now we will open the spreadsheet and worksheet.");
 my $ss = $sheets->open_spreadsheet(name => $spreadsheet_name);
+my $uri = $ss->spreadsheet_uri();
 my $ws0 = $ss->open_worksheet(id => 0);
-end_go("Worksheet is now open.");
+end_go("Worksheet is now open, uri: $uri.");
 
 # resets the spreadsheet. collects up a bunch of batch requests, then
 # gets the spreadsheet to run them. running 'submit_requests' against
@@ -34,7 +35,7 @@ my $col = $ws0->range_col(1);
 my $row = $ws0->range_row(1);
 $col->thaw();
 $row->thaw();
-$ss->submit_requests(requests => [$all, $col, $row]);
+$ss->submit_requests(ranges => [$all, $col, $row]);
 end("'Payroll' spreadsheet should now be blank.");
 
 # load up some sample data without any batch processing.
@@ -81,7 +82,7 @@ $tax->batch_values(values => "=SUM(C2:C4)");
 $salary->batch_values(values => "=SUM(D2:D4)");
 my $rg = $ss->range_group($tax, $salary);
 $rg->submit_values();
-$rg->bold()->italic()->submit_requests();
+$rg->bold()->italic()->bd_solid()->bd_thick('bottom')->submit_requests();
 end("Totals should now be set with formulas.");
 
 message('blue', "We are done, here are some api stats:\n", Dump($ss->stats()));
